// Copyright (c) 2007 Helsinki Institute of Information Technology HIIT
// 
// Permission is hereby granted, free of charge, to any person
// obtaining a copy of this software and associated documentation
// files (the "Software"), to deal in the Software without
// restriction, including without limitation the rights to use,
// copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following
// conditions:
// 
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
// OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
// HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
// WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
// OTHER DEALINGS IN THE SOFTWARE.

using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;

namespace Gazillion
{
    abstract public class FixationRecorder
    {
        // Browser instance ref is required to call javascript
        protected WebBrowser browser;

        // Abstract class
        public FixationRecorder(WebBrowser browser)
        {
            this.browser = browser;
        }

        // Fixation start
        virtual public void FixationStart(int x, int y, int msec)
        {
            // Default is to do nothing
        }

        // Fixation update
        virtual public void FixationUpdate(int x, int y, int msec)
        {
            // Default is to do nothing
        }

        // Fixation end
        virtual public void FixationEnd(int x, int y, int msec)
        {
            // Default is to do nothing
        }

        // Returns a bag of fixations for this trajectory
        abstract public FixationBag GetFixation();

        // Reset fixation store
        abstract public void Clear();

        // Return words which are thought to be relevant
        abstract public ICollection<string> Relevant();

        // Show relevant words on browser
        abstract public void Show();

        // Reset highlights
        public void resetHighlights()
        {
            browser.Document.InvokeScript("resetHighlights");
        }

        // Debug text for application
        public string TrackerText()
        {
            string s = "";
            ICollection<string> words = Relevant();
            foreach (string word in words)
            {
                s += word + "\r\n";
            }
            return s;

        }

        // Retrieve segment Id at x,y from browser
        protected int getWindowSegmentId(int x, int y)
        {
            // segment Id s in the form 'segment:1'
            return getId("getWindowSegmentId", x, y);
        }

        // Retrieve word Id at x,y from browser
        protected int getWindowWordId(int x, int y)
        {
            // word Id s in the form 'word:1'
            return getId("getWindowWordId", x, y);
        }

        // Retrieve word from browser
        protected string getWord(int wordId)
        {
            if (wordId == 0)
                return "";
            return (string)browser.Document.InvokeScript("getWord",
                new string[] { "word:" + wordId });
        }

        // Highlight word for debugging purposes
        protected void highlightWord(int wordId)
        {
            if (wordId == 0)
                return;
            browser.Document.InvokeScript("highlightWord",
                new string[] { "word:" + wordId });
        }

        // Retrieve Id at x,y from browser
        protected int getId(string funName, int x, int y)
        {
            // Id s in the form 'type:1'
            string result = (string)browser.Document.InvokeScript(funName,
                new object[] { x, y });
            if (result == null)
                return 0;
            string[] parts = result.Split(new char[] { ':' });
            try
            {
                return Convert.ToInt32(parts[1]);
            }
            catch
            {
                return 0;
            }
        }


    }
}
